import CloudFlare
from loguru import logger


class Cloudflare(object):
    """
    Prepares cloudflare api client to execute api v4 calls on specific account
    """

    def __init__(self, cf_email: str, cf_key: str):
        self.cf_email = cf_email
        self.cf_key = cf_key
        self.cf_api = CloudFlare.CloudFlare(
            email=self.cf_email, token=self.cf_key)

    @logger.catch
    def purge_all_files(self, zone_name: str):
        zone_id = self.get_zone_id(zone_name)
        purge_cache_result = self.purge_cache(
            zone_id, {'purge_everything': True})
        return purge_cache_result

    @logger.catch
    def purge_files_by_url(self, zone_name: str, urls: list):
        zone_id = self.get_zone_id(zone_name)
        purge_cache_result = self.purge_cache(zone_id, {'files': urls})
        return purge_cache_result

    @logger.catch
    def disable_dns_record_proxy(self, zone_name: str, fqdn: str):
        zone_id = self.get_zone_id(zone_name)
        try:
            record_id = self.get_dns_record_value(zone_name, fqdn)[0]['id']
            return self.cf_api.zones.dns_records.patch(zone_id, record_id, data={'name': fqdn, 'proxied': False})
        except CloudFlare.exceptions.CloudFlareAPIError as e:
            exit('/zones/dns_records %s - %d %s - api call failed' % (fqdn, e, e))

    @logger.catch
    def enable_dns_record_proxy(self, zone_name: str, fqdn: str):
        zone_id = self.get_zone_id(zone_name)
        try:
            record_id = self.get_dns_record_value(zone_name, fqdn)[0]['id']
            return self.cf_api.zones.dns_records.patch(zone_id, record_id, data={'name': fqdn, 'proxied': True})
        except CloudFlare.exceptions.CloudFlareAPIError as e:
            exit('/zones/dns_records %s - %d %s - api call failed' % (fqdn, e, e))

    @logger.catch
    def get_zone_id(self, zone_name: str):
        """

        :param zone_name: domain name
        :return: id of the domain name
        """
        zones_returned = self.cf_api.zones.get(params={'name': f'{zone_name}'})
        zone_id = zones_returned[0]['id']
        return zone_id

    @logger.catch
    def get_dns_record_value(self, zone_name: str, fqdn: str):
        zone_id = self.get_zone_id(zone_name)
        return self.cf_api.zones.dns_records.get(zone_id, params={'name': fqdn, 'type': 'A'})

    @logger.catch
    def purge_cache(self, zone_id: str, payload: dict):
        """
        :param zone_id: cloudflare_zone_id
        :param payload: {'purge_everything': True} or {'files': ['/', '/index.html']}
        :return: {'id': 'zone_id'} or exception
        """
        return self.cf_api.zones.purge_cache.post(zone_id, data=payload)

    @logger.catch
    def create_zone(self, zone_name: str, account_id: str):
        return self.cf_api.zones.post(data={"name": zone_name,
                                            "account": {"id": account_id},
                                            "jump_start": True,
                                            "type": "full"})

    @logger.catch
    def buy_new_domain(self, account_id: str, zone_name: str):
        return self.cf_api.accounts.registrar.domains.get(account_id, zone_name)

    @logger.catch
    def list_domains(self, account_id: str):
        return self.cf_api.accounts.registrar.domains.get(account_id)

    @logger.catch
    def update_dns_record(self, zone_name: str, fqdn: str, value: str):
        zone_id = self.get_zone_id(zone_name)
        try:
            record_id = self.get_dns_record_value(zone_name, fqdn)[0]['id']
            return self.cf_api.zones.dns_records.put(zone_id, record_id,
                                                     data={'type': 'A', 'name': fqdn, 'content': value, 'ttl': 1})
        except CloudFlare.exceptions.CloudFlareAPIError as e:
            exit('/zones/dns_records %s - %d %s - api call failed' % (fqdn, e, e))



from loguru import logger
from namecom import Name


class Namecom(object):
    def __init__(self, username: str, token: str):
        self.username = username
        self.token = token
        self.api_client = Name(username, token)

    @logger.catch
    def create_domain(self, domain_name: str):
        return self.api_client.post('/domains', {'domain': {'domainName': domain_name}})



import requests
import slumber
from loguru import logger


class Dedibox(object):
    def __init__(self, token: str):
        self.token = token
        self.api_session = requests.session()
        self.api_session.headers['Authorization'] = f'Bearer {token}'
        self.api_client = slumber.API('https://api.online.net/api/v1/', session=self.api_session,
                                      append_slash=False)

    @logger.catch
    def is_server_available(self, server_id: str):
        return self.api_client.dedibox.availability(server_id).get()

    # @logger.catch
    def order_server(self, server_id: int, datacenter: int = 5, support: str = 'basic'):
        self.api_client = slumber.API('https://api.online.net/api/v1/', session=self.api_session,
                                      append_slash=True, )
        return self.api_client.dedibox().post({'product': server_id, 'datacenter': datacenter,
                                               'support': support}, )



from atlassian import Jira as Jira_
from loguru import logger


class Jira(object):
    def __init__(self, username: str, token: str):
        self.username = username
        self.token = token
        self.api_client = Jira_(url='https://zadgroup.atlassian.net',
                                username=username, password=token)

    @logger.catch
    def create_project(self, specs: {str, str, str, str, str, str, str}):
        """
        :param specs:
            {
                "description": "AMKMB",
                "leadAccountId": "557058:2c9c859c-34fb-4119-a88c-5ef8ae3c1d0e",
                "projectTemplateKey": "com.pyxis.greenhopper.jira:gh-simplified-scrum-classic",
                "name": "AMKMB",
                "assigneeType": "PROJECT_LEAD",
                "projectTypeKey": "software",
                "key": "AMKMB"
            }
        :return:
            {'self': 'https://zadgroup.atlassian.net/rest/api/2/project/17689',
             'id': 17689,
             'key': 'AMKMB'}
        """
        return self.api_client.create_project_from_raw_json(specs)

    @logger.catch
    def delete_project(self, key: str):
        """

        :param key: Jira project key e.g IQV
        :return: nothing
        """
        return self.api_client.delete_project(key)

    @logger.catch
    def create_project_from_template(self, project_id: str, key: str,
                                     name: str, lead: str, ):
        """

        :param project_id: Project id to use as a template
        :param key: New project key
        :param name: New project name
        :param lead: New project lead username
        :return: json object specs for newly created project
        """
        return self.api_client.create_project_from_shared_template(project_id, key, lead, name)



import sys

import typer
from loguru import logger

from helpo.dedibox import Dedibox

app = typer.Typer()


@logger.catch
@app.command()
def is_server_available(server_id: str,
                        dedi_token: str = ''):
    dedi_api = Dedibox(dedi_token)
    server_info = dedi_api.is_server_available(server_id)
    if server_info['available']:
        typer.echo(server_info)
    else:
        typer.echo("No stock")
        sys.exit(1)


@logger.catch
@app.command()
def order_server(server_id: int, datacenter: int = 5,
                 support: str = 'basic', dedi_token: str = ''):
    dedi_api = Dedibox(dedi_token)
    server_info = dedi_api.order_server(server_id, datacenter, support)
    if server_info['id']:
        typer.echo(server_info)
    else:
        sys.exit(1)


if __name__ == "__main__":
    app()
